unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  Gorilla.Utils.Inventory, Gorilla.UI.Inventory, Gorilla.UI.Inventory.Simple,
  Gorilla.UI.Inventory.Editor, System.Generics.Collections, FMX.DialogService.Sync,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.ListBox, System.ImageList,
  FMX.ImgList, FMX.Layouts, FMX.TabControl, FMX.Menus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    TemplateSelector: TComboBox;
    Button1: TButton;
    TabControl1: TTabControl;
    TbItm_Inventory: TTabItem;
    TbItem_Manufacture: TTabItem;
    Button2: TButton;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Button3: TButton;
    TbItm_Editor: TTabItem;
    TbItm_Sandbox: TTabItem;
    TabControl2: TTabControl;
    Icons: TImageList;
    ManufacturerSelector: TComboBox;
    Button4: TButton;
    PopupMenu1: TPopupMenu;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuItem5Click(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
    procedure MenuItem6Click(Sender: TObject);
    procedure MenuItem7Click(Sender: TObject);
    procedure MenuItem8Click(Sender: TObject);
    procedure MenuItem10Click(Sender: TObject);
    procedure MenuItem11Click(Sender: TObject);
    procedure MenuItem12Click(Sender: TObject);
    procedure MenuItem13Click(Sender: TObject);
  private
    FInventory : TGorillaInventory;
    FSandboxInventory : TGorillaInventory;
    FIFrame    : TInventoryFrame;
    FEFrame    : TInventoryEditorFrame;
    FProcesses : TDictionary<TGorillaInventoryManufacturingProcess, TProgressBar>;

    procedure DoOnInventoryNotify(const ASender : TCustomInventoryControl);
    procedure DoOnManufacturerNotified(
      const AManufacturer : TGorillaInventoryManufacturer;
      const AProcess : TGorillaInventoryManufacturingProcess;
      const AType : TManufacturingNotificationType);
    procedure DoOnManufacturerProduced(
      const AManufacturer : TGorillaInventoryManufacturer;
      const AProcess : TGorillaInventoryManufacturingProcess;
      const AItem : TGorillaInventoryCollectable);
    procedure DoOnInventoryAddManufacturing(const AInventory : TGorillaInventory;
      const AProcess : TGorillaInventoryManufacturingProcess);

    procedure UpdateTemplateList();
    procedure UpdateManufacturerList();

    procedure ResetSandbox();

  public
    property Inventory : TGorillaInventory read FInventory;
    property IFrame : TInventoryFrame read FIFrame;

    procedure UpdateAll();
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

{ $DEFINE CREATEINVENTORY}

uses
  System.TypInfo,
  System.Math.Vectors,
  Gorilla.Utils.Language;

function GenerateRandomName(const APrefix : String) : String;
begin
  result := APrefix + '-' + IntToStr(Random(10000));
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
{$IFDEF CREATEINVENTORY}
var LProdGrp,
    LMainGrp   : TGorillaInventoryGroup;
    LAxe, LPickAxe,
    LWoodTemp,
    LStoneTemp,
    LGrassTemp,
    LChimneyTemp,
    LFireTemp  : TGorillaInventoryItemTemplate;
    LMan       : TGorillaInventoryManufacturer;
    LLangEN_US,
    LLangDE_DE : String;
{$ENDIF}
begin
  // create inventory system
  FInventory := TGorillaInventory.Create(Self);

{$IFDEF CREATEINVENTORY}
  // create group
  LMainGrp := FInventory.AddGroup('Storage');
  LProdGrp := FInventory.AddGroup('Produced');

  LLangEN_US := GetLanguageName(GORILLA_LANG_EN_US);
  LLangDE_DE := GetLanguageName(GORILLA_LANG_DE_DE);

  // create templates
  LAxe := FInventory.AddItemTemplate('Axe');
  LAxe.Group := LMainGrp;
  LAxe.ImageIndex := 4;
  LAxe.Name.Add(LLangEN_US, 'axe');
  LAxe.Name.Add(LLangDE_DE, 'Axt');

  LPickAxe := FInventory.AddItemTemplate('Pickaxe');
  LPickAxe.Group := LMainGrp;
  LPickAxe.ImageIndex := 5;
  LPickAxe.Name.Add(LLangEN_US, 'pickaxe');
  LPickAxe.Name.Add(LLangDE_DE, 'Hacke');

  LWoodTemp := FInventory.AddItemTemplate('Wood');
  LWoodTemp.Group := LMainGrp;
  LWoodTemp.ImageIndex := 0;
  LWoodTemp.IsPooled := false;
  LWoodTemp.AddDependency(LAxe, 1);
  LWoodTemp.Name.Add(LLangEN_US, 'wood');
  LWoodTemp.Name.Add(LLangDE_DE, 'Holz');

  LStoneTemp := FInventory.AddItemTemplate('Stone');
  LStoneTemp.Group := LMainGrp;
  LStoneTemp.ImageIndex := 1;
  LStoneTemp.AddDependency(LPickAxe, 1);
  LStoneTemp.Name.Add(LLangEN_US, 'stone');
  LStoneTemp.Name.Add(LLangDE_DE, 'Stein');

  LGrassTemp := FInventory.AddItemTemplate('Grass');
  LGrassTemp.Group := LMainGrp;
  LGrassTemp.ImageIndex := 2;
  LGrassTemp.Name.Add(LLangEN_US, 'grass');
  LGrassTemp.Name.Add(LLangDE_DE, 'Gras');

  LFireTemp := FInventory.AddItemTemplate('Fire');
  LFireTemp.Group := LProdGrp;
  LFireTemp.ImageIndex := 3;
  LFireTemp.Name.Add(LLangEN_US, 'fire');
  LFireTemp.Name.Add(LLangDE_DE, 'Feuer');

  LChimneyTemp := FInventory.AddItemTemplate('Chimney');
  LChimneyTemp.Group := LProdGrp;
  LChimneyTemp.ImageIndex := 6;
  LChimneyTemp.DowngradeItem := LFireTemp;
  LChimneyTemp.Name.Add(LLangEN_US, 'chimney');
  LChimneyTemp.Name.Add(LLangDE_DE, 'Kamin');

  LFireTemp.UpgradeItem := LChimneyTemp;

  // create manufacturer: in this example we want to make campfire
  // by 3 pieces of wood, 2 fire stones and 1 pieces of dry grass
  // the result item will be the campfire item
  // this should be added to the second group "Produced"
  LMan := FInventory.AddManufacturer('Campfire');
  LMan.TimeNeeded := 10000;
  LMan.AddIngredient(LWoodTemp, 3);
  LMan.AddIngredient(LStoneTemp, 2);
  LMan.AddIngredient(LGrassTemp, 1);
  LMan.ResultItem.Template := LFireTemp;
{$ENDIF}

  // create inventory frame
  FIFrame := TInventoryFrame.Create(TbItm_Inventory);
  FIFrame.Parent := TbItm_Inventory;
  FIFrame.Align  := TAlignLayout.Client;
  FIFrame.Inventory := FInventory;
//  FIFrame.Images := ImageList1;
  FIFrame.Margins.Rect := RectF(8, 8, 8, 8);
  FIFrame.OnClickItem := DoOnInventoryNotify;

  FProcesses := TDictionary<TGorillaInventoryManufacturingProcess, TProgressBar>.Create();

  // editor frame
  FEFrame := TInventoryEditorFrame.Create(TbItm_Editor);
  FEFrame.Parent := TbItm_Editor;
  FEFrame.Align := TAlignLayout.Client;
  FEFrame.Inventory := FInventory;

  UpdateAll();
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FProcesses);
end;

procedure TForm1.DoOnInventoryNotify(const ASender : TCustomInventoryControl);

  function GetAbsolutePosition(AParent : TFmxObject) : TPointF;
  begin
    if Assigned(AParent) then
    begin
      if (AParent is TControl) then
        Result := GetAbsolutePosition(AParent.Parent) + TControl(AParent).AbsoluteRect.TopLeft
      else
        Result := GetAbsolutePosition(AParent.Parent);
    end
    else Result := TPointF.Zero;
  end;

var LPos : TPointF;
begin
  if (ASender.Instance is TGorillaInventoryCollectable) then
  begin
    PopUpMenu1.PopupComponent := ASender;
    PopUpMenu1.TagObject := ASender.Instance;
    LPos := GetAbsolutePosition(ASender);
    PopUpMenu1.Popup(LPos.X, LPos.Y);
  end;
end;

procedure TForm1.MenuItem10Click(Sender: TObject);
var LGrp : TGorillaInventoryGroup;
    LId : String;
begin
  // new group ...
  LId  := GenerateRandomName('Group');
  LGrp := FInventory.AddGroup(LId) as TGorillaInventoryGroup;
  FEFrame.UpdateTreeView();
  FEFrame.LoadItemToEdit(LGrp);
end;

procedure TForm1.MenuItem11Click(Sender: TObject);
var LItem : TGorillaInventoryItemTemplate;
    LId : String;
begin
  // new template ...
  LId  := GenerateRandomName('Template');
  LItem := FInventory.AddItemTemplate(LId) as TGorillaInventoryItemTemplate;
  FEFrame.UpdateTreeView();
  FEFrame.LoadItemToEdit(LItem);
end;

procedure TForm1.MenuItem12Click(Sender: TObject);
var LItem : TGorillaInventoryManufacturer;
    LId : String;
begin
  // new manufacturer ...
  LId  := GenerateRandomName('Manufacturer');
  LItem := FInventory.AddManufacturer(LId) as TGorillaInventoryManufacturer;
  FEFrame.UpdateTreeView();
  FEFrame.LoadItemToEdit(LItem);
end;

procedure TForm1.MenuItem13Click(Sender: TObject);
var LItem : TGorillaInventoryCollectable;
    LId : String;
begin
  // new collected item ...
  if (FInventory.ItemTemplates.Count > 0) then
  begin
    LId  := GenerateRandomName('Collectable');
    LItem := FInventory.CollectedItems.Add() as TGorillaInventoryCollectable;
    LItem.Id := LId;
    LItem.Template := FInventory.ItemTemplates.FindItemID(0) as TGorillaInventoryItemTemplate;
    FEFrame.UpdateTreeView();
    FEFrame.LoadItemToEdit(LItem);
  end
  else ShowMessage('Failed to create collected item - add at least one item template');
end;

procedure TForm1.MenuItem2Click(Sender: TObject);
begin
  // open inventory file
  if OpenDialog1.Execute() then
  begin
    FInventory.LoadFromFile(OpenDialog1.FileName);
    UpdateAll();
  end;
end;

procedure TForm1.MenuItem3Click(Sender: TObject);
begin
  // save as inventory file
  if SaveDialog1.Execute then
    FInventory.SaveToFile(SaveDialog1.FileName);
end;

procedure TForm1.MenuItem5Click(Sender: TObject);
begin
  Application.Terminate();
end;

procedure TForm1.MenuItem6Click(Sender: TObject);
var LColl  : TGorillaInventoryCollectable;
    LCount : Integer;
begin
  // drop the item
  LColl := PopUpMenu1.TagObject as TGorillaInventoryCollectable;
  if not Assigned(LColl) then
    Exit;

  LCount := -1;
  if LColl.Template.IsPooled then
  begin
    if (FMX.DialogService.Sync.TDialogServiceSync.MessageDialog(
      'Would you like to drop all instances of that item?',
      TMsgDlgType.mtConfirmation, [TMsgDlgBtn.mbYes, TMsgDlgBtn.mbNo], TMsgDlgBtn.mbYes, 0) = mrNo) then
    begin
      LCount := 1;
    end;
  end;

  FSandboxInventory.DropCollectedItem(LColl, LCount);
  FIFrame.UpdateInventory();
end;

procedure TForm1.MenuItem7Click(Sender: TObject);
var LColl : TGorillaInventoryCollectable;
begin
  // upgrade the item
  LColl := PopUpMenu1.TagObject as TGorillaInventoryCollectable;
  if not Assigned(LColl) then
    Exit;

  if LColl.IsUpgradable() then
    FSandboxInventory.Upgrade(LColl)
  else
    ShowMessage('Collected item not upgradable!');
end;

procedure TForm1.MenuItem8Click(Sender: TObject);
var LColl : TGorillaInventoryCollectable;
begin
  // downgrade the item
  LColl := PopUpMenu1.TagObject as TGorillaInventoryCollectable;
  if not Assigned(LColl) then
    Exit;

  if LColl.IsDowngradable() then
    FSandboxInventory.Downgrade(LColl)
  else
    ShowMessage('Collected item not downgradable!');
end;

procedure TForm1.UpdateAll();
begin
  UpdateTemplateList();
  UpdateManufacturerList();

  // update image list
  FIFrame.UpdateInventory();
  FEFrame.UpdateVisualItems();
  FEFrame.UpdateTreeView();
end;

procedure TForm1.UpdateTemplateList();
var i : Integer;
    LTemp : TGorillaInventoryItemTemplate;
begin
  TemplateSelector.Clear();

  if Assigned(FSandboxInventory) then
  for i := 0 to (FSandboxInventory.ItemTemplates.Count - 1) do
  begin
    LTemp := FSandboxInventory.ItemTemplates.Items[i] as TGorillaInventoryItemTemplate;
    TemplateSelector.Items.AddObject(LTemp.Id, LTemp);
  end;
end;

procedure TForm1.UpdateManufacturerList();
var i : Integer;
    LTemp : TGorillaInventoryManufacturer;
begin
  ManufacturerSelector.Clear();

  if Assigned(FSandboxInventory) then
  begin
    for i := 0 to (FSandboxInventory.Manufacturers.Count - 1) do
    begin
      LTemp := FSandboxInventory.Manufacturers.Items[i] as TGorillaInventoryManufacturer;
      ManufacturerSelector.Items.AddObject(LTemp.Id, LTemp);
    end;
  end;
end;

procedure TForm1.ResetSandbox();
begin
  if Assigned(FSandboxInventory) then
    FreeAndNil(FSandboxInventory);

  FSandboxInventory := FInventory.Clone();
  FSandboxInventory.OnAddManufacturing := DoOnInventoryAddManufacturing;
  FSandboxInventory.Language := GORILLA_LANG_DE_DE;

  FIFrame.Inventory := FSandboxInventory;

  UpdateTemplateList();
  UpdateManufacturerList();
end;

procedure TForm1.TabControl1Change(Sender: TObject);
begin
  if (TabControl1.ActiveTab = TbItm_Sandbox) then
  begin
    ResetSandbox();
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var LIdx  : Integer;
    LTemp : TGorillaInventoryItemTemplate;
    LState : TGorillaInventoryCollectState;
begin
  // collect a specific item
  LIdx := TemplateSelector.ItemIndex;
  if (LIdx < 0) then
    Exit;

  LTemp := TemplateSelector.Items.Objects[LIdx] as TGorillaInventoryItemTemplate;
  FSandboxInventory.Collect(LTemp, LState);
  if (LState <> TGorillaInventoryCollectState.Collected) then
    ShowMessage('Not collected: ' +
      GetEnumName(TypeInfo(TGorillaInventoryCollectState), Ord(LState)))
  else
  begin
    UpdateAll();
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
var LProc : TGorillaInventoryManufacturingProcess;
begin
  // start manufacturing process
  LProc := FSandboxInventory.Manufacture(
    FSandboxInventory.CollectedItems.ToArray()
    );

  if not Assigned(LProc) then
  begin
    ShowMessage('Some ingredients missing!');
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  // update inventory view
  UpdateAll();
end;

procedure TForm1.Button4Click(Sender: TObject);
var LMan  : TGorillaInventoryManufacturer;
    LProc : TGorillaInventoryManufacturingProcess;
begin
  // manufacture with a specific manufacturer
  if Assigned(ManufacturerSelector.Selected) then
  begin
    LMan := ManufacturerSelector.Selected.Data as TGorillaInventoryManufacturer;
    if Assigned(LMan) then
    begin
      LProc := FSandboxInventory.Manufacture(LMan, FSandboxInventory.CollectedItems.ToArray());
      if not Assigned(LProc) then
      begin
        ShowMessage('Some ingredients missing!');
      end;
    end;
  end
  else ShowMessage('Please select a manufacturer at first.');
end;

procedure TForm1.DoOnManufacturerNotified(
  const AManufacturer: TGorillaInventoryManufacturer;
  const AProcess: TGorillaInventoryManufacturingProcess;
  const AType: TManufacturingNotificationType);
var LBar  : TProgressBar;
    LLbl  : TLabel;
    LKind : String;
begin
  case AType of
    mntStarted:
      begin
        UpdateAll();

        // create manufacturing process progressbar
        LBar := TProgressBar.Create(TbItem_Manufacture);
        LBar.Parent := TbItem_Manufacture;
        LBar.Align  := TAlignLayout.Top;
        LBar.Height := 32;
        LBar.Margins.Rect := RectF(8, 8, 8, 8);
        LBar.TagObject := AProcess;
        LBar.Max := AManufacturer.TimeNeeded;

        LLbl := TLabel.Create(LBar);
        LLbl.Parent := LBar;
        LLbl.Align  := TAlignLayout.Client;
        LLbl.Margins.Rect := RectF(4, 4, 4, 4);

        case AProcess.Kind of
          NewItem     : LKind := ' (New)';
          Upgrading   : LKind := ' (Upgrading)';
          Downgrading : LKind := ' (Downgrading)';

          else LKind := ' (?)';
        end;

        LLbl.Text := AManufacturer.GetDescriptionString() + LKind;

        FProcesses.Add(AProcess, LBar);

        // switch to manufacturing process
        TabControl2.ActiveTab := TbItem_Manufacture;
      end;

    mntPaused: ;

    mntAborted :
      begin
        // destroy progress bar
        if FProcesses.TryGetValue(AProcess, LBar) then
        begin
          FProcesses.Remove(AProcess);
          LBar.DisposeOf();
        end;
      end;

    mntCompleted:
      begin
        // destroy progress bar
        if FProcesses.TryGetValue(AProcess, LBar) then
        begin
          FProcesses.Remove(AProcess);
          LBar.DisposeOf();
        end;
      end;

    mntProcessing:
      begin
        // update progressbar
        if FProcesses.TryGetValue(AProcess, LBar) then
        begin
          LBar.Value := AProcess.Progress;
        end;
      end;
  end;
end;

procedure TForm1.DoOnManufacturerProduced(
  const AManufacturer: TGorillaInventoryManufacturer;
  const AProcess: TGorillaInventoryManufacturingProcess;
  const AItem: TGorillaInventoryCollectable);
begin
  // new item produced - we need to update the inventory frame
  UpdateAll();
  TabControl2.ActiveTab := TbItm_Inventory;
end;

procedure TForm1.DoOnInventoryAddManufacturing(const AInventory : TGorillaInventory;
  const AProcess : TGorillaInventoryManufacturingProcess);
begin
  AProcess.Manufacturer.OnNotify := DoOnManufacturerNotified;
  AProcess.Manufacturer.OnProduced := DoOnManufacturerProduced;
end;

end.
